; Program demonstracyjny, obsuga moduu LCD (HD44780)
; Rafa Baranowski (C) 2004

.INCLUDE "m16def.inc"		; SEGMENT DEKLARACJI

.EQU	SYS_FREQ	= 1		; czstotliwo pracy w MHz

.EQU	K_KLAW	= DDRB	; rejestr kierunku portu klawiatury
.EQU	O_KLAW	= PORTB	; rejestr wyjciowy portu klawiatury
.EQU	I_KLAW	= PINB	; rejestr wejciowy portu klawiatury
.EQU	K_KL_WCISN	= DDRD	; rejestr kierunku linii "kl. wcinity"
.EQU	O_KL_WCISN	= PORTD	; rejestr wyjciowy linii "kl. wcinity"
.EQU	I_KL_WCISN	= PIND	; rejestr wejciowy linii "kl. wcinity"
.EQU	KL_WCISN	= 2		; nr linii dla sygnau "kl. wcinity"

.EQU K_LCD		= DDRD	; rejestr kierunku portu wywietlacza
.EQU O_LCD		= PORTD	; rejestr wyjciowy portu wywietlacza
.EQU LCD_RS		= 0		; nr linii dla sygnau RS
.EQU LCD_EN		= 1		; nr linii dla sygnau EN
.EQU LCD_DB4	= 4		; nr linii dla sygnau DB4
.EQU LCD_DB5	= 5		; nr linii dla sygnau DB5
.EQU LCD_DB6	= 6		; nr linii dla sygnau DB6
.EQU LCD_DB7	= 7		; nr linii dla sygnau DB7


.DSEG					; SEGMENT DANYCH
.ORG 0x0060				; pocztek pamici SRAM
Klaw_inf:		.BYTE	1	; zmienna stanu klawiatury matrycowej

.CSEG					; SEGMENT PROGRAMU
.ORG	0				; wektor zerowania
	jmp	Reset			; skok do procedury inicjalizujcej

.ORG	INT0addr			; wektor przerwania INT0
	jmp	Odczyt_klawiatury	; skok do procedury odczytujcej stan klawiatury

					; deklaracje staych w pamici programu
Powitanie:	.db	"Demonstracja LCD", ZNAK_KONCA
Tekst:	.db	POL_L, "a", POL_N, "cuch", ZNAK_KONCA


Reset:				; podprogram inicjalizujcy
	ldi	R17, high(RAMEND)
	ldi	R16, low(RAMEND)
	out	SPH, R17
	out	SPL, R16		; inicjalizacja wskanika stosu

	clr	R16
	sts	Klaw_inf, R16	; inicjalizacja zmiennej Klaw_inf

	ldi	R16, 1<<CS00
	out	TCCR0, R16		; licznik 0. zlicza z czstotliwoci systemow

	rcall	Ini_klawiatury	; inicjalizacja klawiatury matrycowej
	rcall	Ini_LCD		; inicjalizacja wywietlacza LCD

	ldi	ZH, high(Polskie_znaki<<1)
	ldi	ZL, low(Polskie_znaki<<1)
	rcall	Zapisz_CGRAM	; zapisanie pamici CGRAM ksztatami polskich
					;   znakw diakrytycznych

	ldi	ZH, high(Powitanie<<1)
	ldi	ZL, low(Powitanie<<1)
	rcall	Pokaz_ciag_LCD	; wywietlenie powitania ("Demonstracja LCD")

	ldi	R17, 100
	ldi	R16, 10
	rcall	Czekaj_ms		; opnienie o 1 s

	rcall	Czysc_LCD		; czyszczenie wywietlacza

	sei				; odblokowanie przerwa


Petla:				; ptla gwna programu
	lds	R16, Klaw_inf	
	sbrs	R16, 7
	rjmp	Petla			; oczekuj na wcinicie klawisza

	lds	R16, Klaw_inf	; zapisz warto klawisza w zmiennej R16
	cbr	R16, 1<<7		;   i wyzeruj bit informujcy o jego wciniciu

					; blok wyboru podprogramu
	cpi	R16, 0x00		; klawisz "0" - wczanie kursora bez migania,
	brne	Petla_1		;		aktywacja wywietlacza
	
	ldi	R16, (1<<LCD_STER)|(1<<LCD_SW_ON)|(1<<LCD_SK_ON)|(1<<LCD_SM_OFF)
	rcall	Instrukcja_LCD
	
	rjmp	Koniec

Petla_1:
	cpi	R16, 0x01		; klawisz "1" - przesuwanie okna w lewo
	brne	Petla_2

	ldi	R16, (1<<LCD_PRZES)|(1<<LCD_P_WYS)|(1<<LCD_LEWO)
	rcall	Instrukcja_LCD

	rjmp	Koniec

Petla_2:
	cpi	R16, 0x02		; klawisz "2" - czyszczenie wywietlacza
	brne	Petla_3
	
	rcall	Czysc_LCD

	rjmp	Koniec

Petla_3:
	cpi	R16, 0x03		; klawisz "3" - przesuwanie okna w prawo
	brne	Petla_4

	ldi	R16, (1<<LCD_PRZES)|(1<<LCD_P_WYS)|(1<<LCD_PRAWO)
	rcall	Instrukcja_LCD

	rjmp	Koniec

Petla_4:
	cpi	R16, 0x04		; klawisz "4" - przesuwanie kursora w lewo
	brne	Petla_5

	ldi	R16, (1<<LCD_PRZES)|(1<<LCD_P_KUR)|(1<<LCD_LEWO)
	rcall	Instrukcja_LCD

	rjmp	Koniec

Petla_5:
	cpi	R16, 0x05		; klawisz "5" - powrt kursora
	brne	Petla_6

	rcall	Wroc_LCD
	
	rjmp	Koniec

Petla_6:
	cpi	R16, 0x06		; klawisz "6" - przesuwanie kursora w prawo
	brne	Petla_7

	ldi	R16, (1<<LCD_PRZES)|(1<<LCD_P_KUR)|(1<<LCD_PRAWO)
	rcall	Instrukcja_LCD

	rjmp	Koniec

Petla_7:
	cpi	R16, 0x07		; klawisz "7" - tryb dekrementacji
	brne	Petla_8

	ldi	R16, (1<<LCD_TRYB)|(1<<LCD_T_DEC)
	rcall	Instrukcja_LCD	

	rjmp	Koniec

Petla_8:
	cpi	R16, 0x08		; klawisz "8" - przejcie do wiersza dolnego
	brne	Petla_9

	
	ldi	R16, 40|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD

	rjmp	Koniec

Petla_9:	
	cpi	R16, 0x09		; klawisz "9" - tryb inkrementacji z przesuwaniem
	brne	Petla_10

	ldi	R16, (1<<LCD_TRYB)|(1<<LCD_T_INC)|(1<<LCD_T_PRZES)
	rcall	Instrukcja_LCD

	rjmp	Koniec

Petla_10:
	cpi	R16, 0x0A		; klawisz "A" - wywietlanie acucha
	brne	Petla_11

	ldi	ZH, high(Tekst<<1)
	ldi	ZL, low(Tekst<<1)
	rcall	Pokaz_ciag_LCD

	rjmp	Koniec

Petla_11:
	cpi	R16, 0x0B		; klawisz "B" - wywietlanie zawartoci TCNT0
	brne	Petla_12

	in	R16, TCNT0
	rcall	Pokaz_bajt_LCD

	rjmp	Koniec

Petla_12:
	cpi	R16, 0x0C		; klawisz "C" - generacja 3. spacji
	brne	Petla_13
	
	ldi	R16, 3
	rcall	Spacja_LCD

	rjmp	Koniec

Petla_13:
	cpi	R16, 0x0D		; klawisz "D" - deaktywacja wywietlacza
	brne	Petla_14

	ldi	R16, (1<<LCD_STER)|(1<<LCD_SW_OFF)
	rcall	Instrukcja_LCD

	rjmp	Koniec

Petla_14:
	cpi	R16, 0x0E		; klawisz "#" - kursor migajcy,
	brne	Petla_15		;		aktywacja wywietlacza

	ldi	R16, (1<<LCD_STER)|(1<<LCD_SW_ON)|(1<<LCD_SK_OFF)|(1<<LCD_SM_ON)
	rcall	Instrukcja_LCD

	rjmp	Koniec

Petla_15:				; klawisz "*" - inicjalizacja wywietlacza

	rcall	Ini_LCD

Koniec:
	lds	R16, Klaw_inf
	sbrc	R16, 7
	rjmp	Koniec		; oczekiwanie na puszczenie klawisza

	rjmp	Petla			; zaptl program


					; DOCZANE PODPROGRAMY
.INCLUDE "czekaj_ms.inc"	;   oczekiwanie milisekundowe
.INCLUDE "czekaj_us.inc"	;   oczekiwanie mikrosekundowe
.INCLUDE "klaw_mat.inc"		;   obsuga klawiatury matrycowej
.INCLUDE "lcd.inc"		;   obsuga wywietlacza LCD
